import org.codehaus.groovy.scriptom.ActiveXObject
import org.codehaus.groovy.scriptom.Scriptom

def exitcode = 0
def failmode = "fast"
def buglist = [] as List

try {
	Scriptom.inApartment{
	    def testDirector = null

        final def stepProps = new Properties();
        final def inputPropsFile = new File(args[0]);
        final def inputPropsStream = null;
        try {
            inputPropsStream = new FileInputStream(inputPropsFile);
            stepProps.load(inputPropsStream);
        }
        catch (IOException e) {
            throw new RuntimeException(e);
        }
	
	    def serverUrl = stepProps['serverUrl']

        println "********************************************"
        println "SERVER URL : $serverUrl";
        println "********************************************"

	    def username = stepProps['username']

        println "********************************************"
        println "SERVER Username : $username";
        println "********************************************"

		def password = stepProps['password'] ? stepProps['password'] : stepProps['passwordscript']
		 
	    def domain = stepProps['qcdomain']

        println "********************************************"
        println "SERVER domain : $domain";
        println "********************************************"

	    def project = stepProps['qcproject']

        println "********************************************"
        println "SERVER project : $project";
        println "********************************************"

	    def projectkey = stepProps['projectkey']        

        println "********************************************"
        println "Projectkey : $projectkey";
        println "********************************************"

	    def issuePropsStr = stepProps['props']
		def capturemode = stepProps['capturemode']
		def bugpattern = stepProps['bugpattern']
		failmode = stepProps['failmode']
		
		// parse the additional fields into a Properties object
		def updateFields = new Properties()
		if (issuePropsStr != null) {
			try {
	            updateFields.load(new ByteArrayInputStream(issuePropsStr.getBytes()))
	        }
	        catch (Exception e) {
	            throw new Exception("Additional fields formatted incorrectly: " + e.getMessage())
	        }
		}
	
		// add the defined fields that can be updated to the Properties object
		if (stepProps['assignee']) {
			updateFields.setProperty("AssignedTo", stepProps['assignee'])
		}
		if (stepProps['summary']) {
			updateFields.setProperty("Summary", stepProps['summary'])
		}
		if (stepProps['priority']) {
			updateFields.setProperty("Priority", stepProps['priority'])
		}
		if (stepProps['status']) {
			updateFields.setProperty("Status", stepProps['status'])
		}
		if (stepProps['severity']) {
			updateFields.setProperty("BG_SEVERITY", stepProps['severity'])
		}
		if (stepProps['qcprojectkey']) {
			updateFields.setProperty("Project", stepProps['qcprojectkey'])
		}
		if (stepProps['detectedby']) {
			updateFields.setProperty("DetectedBy", stepProps['detectedby'])
		}
	
		try {
			println 'Creating connection to Quality Center server'
		    testDirector = new ActiveXObject("TDApiOle80.TDConnection")
		    testDirector.InitConnectionEx(serverUrl)
		    testDirector.Login(username, password)
		    testDirector.Connect(domain, project)
		    println 'Connection established'
			
			def reqNumbugs = 0
			def actNumbugs = 0
			def bugFactory = testDirector.BugFactory
			
			println 'Defect Ids to Update: ' + bugpattern
			reqNumbugs = bugpattern.split(",").length
			
			def defectIdQuery = bugpattern.replace(",", " OR ")
			def bugFilter = bugFactory.Filter
			bugFilter.Filter.BG_BUG_ID = defectIdQuery
		    def tmplist = bugFilter.NewList()
			for (bug in tmplist) {
				println "Found defect " + bug['id'] + " - " + bug['Summary']
				buglist.add(bug)
				actNumbugs++
			}
			
			println "Found $actNumbugs of $reqNumbugs defects in Quality Center."
			
			if (reqNumbugs != actNumbugs) {
				if (failmode == "slow") {
					println "Fail mode is 'Fail' and not all requested defects were found in Quality Center. Scheduling failure after the found defects have been updated."
					exitcode = 1
				}
				else if (failmode == "warn") {
					println "Fail mode is 'Warn' and not all requested defects were found in Quality Center. Only logging a warning."
				}
				else { // default is fast
					println "Fail mode is 'Fail-fast' and not all requested defects were found in Quality Center. Failing now."
					throw new Exception("Not all requested defects were found in Quality Center and fail mode is 'Fail-fast'")
				}
			}
		    
		    for (bug in buglist) {
				println 'Updating defect ' + bug['id']
		        for (updateField in updateFields.entrySet() ) {
					def name = updateField.getKey()
					def value = updateField.getValue()
					
					if (value != null && value.length() > 0) {
					    println "Updating defect " + bug['id'] + " $name to $value"
						switch (name) {
							case "AssignedTo":
							case "Priority":
							case "Status":
							case "Summary":
							case "Project":
							case "DetectedBy":
								bug[name] = value
								break;
							default:
								bug.Field[name] = value
						}
					}
		        } // end for updateField in updateFields entry set
		        
				println 'Posting changes to defect ' + bug['id']
				bug.Post()
				bug.UnlockObject()
				println 'Post complete'
				
		    } // end for bug in buglist
	    }
	    finally {
	    	if (testDirector) {
		        try {
			        if (testDirector.Connected()) {
			            testDirector.Disconnect()
			        }
			        if (testDirector.LoggedIn()) {
			            testDirector.Logout()
			        }
		            testDirector.ReleaseConnection()
		            testDirector.safeRelease()
		        }
		        catch (Exception e) {}
	        }
	    }
	}
}
catch (Exception e) {
	println e.getMessage()
	exitcode = 1
}
com.jacob.com.ComThread.Release();

if (exitcode) {
	System.exit(exitcode)
}
